@@ -1,4 +1,4 @@
-from .orthanc_cloner import OrthancCloner
+from .orthanc_cloner import OrthancCloner, ClonerMode
 from .orthanc_folder_importer import *
 from .orthanc_monitor import OrthancMonitor
 from .orthanc_test_db_populator import OrthancTestDbPopulator
